@@ -21,8 +21,9 @@
 #include "coreirclisthelper.h"
 
 #include "corenetwork.h"
-#include "userinputhandler.h"
+#include "coreuserinputhandler.h"
 
+INIT_SYNCABLE_OBJECT(CoreIrcListHelper)
 QVariantList CoreIrcListHelper::requestChannelList(const NetworkId &netId, const QStringList &channelFilters) {
   if(_finishedChannelLists.contains(netId))
     return _finishedChannelLists.take(netId);
