@@ -23,8 +23,15 @@
 #include <QDrag>
 #include <QGraphicsSceneMouseEvent>
 #include <QMenu>
+#include <QMenuBar>
 #include <QPersistentModelIndex>
 
+#ifdef HAVE_KDE
+#  include <KMenuBar>
+#else
+#  include <QMenuBar>
+#endif
+
 #ifdef HAVE_WEBKIT
 #  include <QWebView>
 #endif
@@ -598,6 +605,9 @@ void ChatScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
     // no item -> default scene actions
     GraphicalUi::contextMenuActionProvider()->addActions(&menu, filter(), BufferId());
 
+  if (QtUi::mainWindow()->menuBar()->isHidden())
+    menu.addAction(QtUi::actionCollection("General")->action("ToggleMenuBar"));
+
   menu.exec(event->screenPos());
 
 }
