@@ -20,6 +20,7 @@
 
 #include "abstractbuffercontainer.h"
 #include "client.h"
+#include "clientbacklogmanager.h"
 #include "networkmodel.h"
 
 AbstractBufferContainer::AbstractBufferContainer(QWidget *parent)
@@ -93,5 +94,10 @@ void AbstractBufferContainer::setCurrentBuffer(BufferId bufferId) {
   _currentBuffer = bufferId;
   showChatView(bufferId);
   Client::networkModel()->clearBufferActivity(bufferId);
+  Client::backlogManager()->checkForBacklog(bufferId);
   setFocus();
+
+  if(bufferId.isValid() && _chatViews.contains(bufferId)) {
+    Client::setBufferLastSeenMsg(bufferId, _chatViews[bufferId]->lastMsgId());
+  }
 }
